from browser import document, html, window
import math

canvas = document["vectorCanvas"]
ctx = canvas.getContext("2d")

# Initial vectors
vector_a = [-100, 100]
vector_b = [200, 150]
origin = [400, 300]
scalar = 2
SCALE = 1
TUNE = 40
dragging_a = False
dragging_b = False
law = ""

def draw_vectors():
    ctx.clearRect(0, 0, canvas.width, canvas.height)
    draw_vector(origin, vector_a, "red", text="a")
    draw_vector(origin, vector_b, "blue", text="b")

def draw_vector(origin, vec, color, style=False, text=""):
    ctx.beginPath()
    if style:
        ctx.setLineDash([10,5])
    else:
        ctx.setLineDash([])
    ctx.moveTo(origin[0], origin[1])
    x, y = (origin[0] + vec[0], origin[1] - vec[1])
    ctx.lineTo(x,y)
    ctx.strokeStyle = color
    ctx.lineWidth = 3
    ctx.stroke()

    headlen = 20
    angle = math.atan2(vec[1], vec[0]) #atan2 outputs from pi to -pi
    
    ctx.moveTo(origin[0] + vec[0], origin[1] -vec[1])
    ctx.lineTo(origin[0] + vec[0]-headlen*math.cos(angle-math.pi/6), origin[1] -vec[1]+headlen*math.sin(angle-math.pi/6))
    ctx.moveTo(origin[0] + vec[0], origin[1] -vec[1])
    ctx.lineTo(origin[0] + vec[0]-headlen*math.cos(angle+math.pi/6), origin[1] -vec[1]+headlen*math.sin(angle+math.pi/6))
    ctx.stroke()

    ctx.font = "1.8em sans-serif"
    ctx.fillStyle= color
    ctx.fillText(text, x-(vec[0]/2)-10-10*math.cos(angle-math.pi/6), y+(vec[1]/2)-20-10*math.sin(angle-math.pi/6))

def vector_add(e):
    global law
    result = [vector_a[i] + vector_b[i] for i in range(2)]
    draw_result_vector(result, text="a+b")
    if law == "Tri":
        ctx.font = "1.8em sans-serif"
        ctx.fillStyle= "orange"
        ctx.fillText("Triangle Law", 20, 30)
        draw_vector([origin[0] + vector_b[0], origin[1]- vector_b[1]], vector_a, "orange", style=True, text="a")
    elif law == "Par":
        ctx.font = "1.8em sans-serif"
        ctx.fillStyle= "orange"
        ctx.fillText("Parellologram Law", 20, 30)
        draw_vector([origin[0] + vector_b[0], origin[1]- vector_b[1]], vector_a, "orange", style=True, text="a")
        draw_vector([origin[0] + vector_b[0], origin[1]- vector_b[1]], vector_a, "orange", style=True, text="a'")
        draw_vector([origin[0] + vector_a[0], origin[1]- vector_a[1]], vector_b, "orange", style=True, text="b'")
    #document["result"].text = f"Vector A + Vector B = {result}, mouse = {get_mouse_pos(e)}, law = {law}"

def vector_subtract(e):
    result = [vector_a[i] - vector_b[i] for i in range(2)]
    draw_result_vector(result)
    document["result"].text = f"Vector A - Vector B = {result}"

def vector_scalar_multiply(e):
    result = [scalar * vector_a[i] for i in range(2)]
    draw_result_vector(result)
    document["result"].text = f"Scalar * Vector A = {result}"

def vector_mag(vec):
    return math.sqrt(sum(vec[i]**2 for i in range(2)))

def vector_dot(v,w):
    return sum(vector_a[i] * vector_b[i] for i in range(2))

def vector_angle(e):
    dot_product = vector_dot(vector_a, vector_b) 
    magnitude_a = vector_mag(vector_a)
    magnitude_b = vector_mag(vector_b)
    cosine_angle = dot_product / (magnitude_a * magnitude_b)
    angle = math.acos(cosine_angle) * (180 / math.pi)
    document["result"].text = f"Angle between A and B = {angle:.2f} degrees"

def draw_result_vector(result, origin=origin, text=""):
    draw_vectors()
    draw_vector(origin, result, "green", text=text)

def get_mouse_pos(evt):
    global SCALE
    rect = canvas.getBoundingClientRect()
    x = evt.clientX - rect.left
    y = evt.clientY - rect.top
    x*= canvas.width/(rect.right-rect.left)
    y*= canvas.height/(rect.bottom-rect.top)
    return [x, y]

def on_mouse_down(evt):
    global dragging_a, dragging_b
    mouse_pos = get_mouse_pos(evt)
    if is_near_vector(mouse_pos, vector_a):
        dragging_a = True
    elif is_near_vector(mouse_pos, vector_b):
        dragging_b = True

def on_mouse_move(evt):
    global vector_a, vector_b
    if dragging_a or dragging_b:
        mouse_pos = get_mouse_pos(evt)
        if dragging_a:
            vector_a = [mouse_pos[0] - origin[0], origin[1] - mouse_pos[1]]
        elif dragging_b:
            vector_b = [mouse_pos[0] - origin[0], origin[1] - mouse_pos[1]]
        draw_vectors()

def on_mouse_up(evt):
    global dragging_a, dragging_b
    dragging_a = False
    dragging_b = False

def is_near_vector(mouse_pos, vector):
    global TUNE
    vec_pos = [origin[0] + vector[0], origin[1] - vector[1]]
    distance = math.sqrt((mouse_pos[0] - vec_pos[0])**2 + (mouse_pos[1] - vec_pos[1])**2)
    return distance < TUNE

def tri_law(event):
    global law
    if law == "":
        law = "Tri"
    elif law == "Tri":
        law = "Par"
    else:
        law = ""

canvas.bind("mousedown", on_mouse_down)
canvas.bind("mousemove", on_mouse_move)
canvas.bind("mouseup", on_mouse_up)

document["vectorCanvas"].bind("mousemove", vector_add)
#document["subtractBtn"].bind("click", vector_subtract)
#document["scalarMultiplyBtn"].bind("click", vector_scalar_multiply)
#document["angleBtn"].bind("click", vector_angle)
document["triBtn"].bind("click", tri_law)
document["triBtn"].bind("click", vector_add)

draw_vectors()